using System.Diagnostics;
using System.Drawing.Drawing2D;
using System.Management;
using System.Reflection;
using System.Runtime.InteropServices;

namespace Oyasumi
{
    public partial class Form1 : Form
    {
        protected override CreateParams CreateParams
        {
            get
            {
                const int WS_EX_LAYERED = 0x00080000;

                CreateParams cp = base.CreateParams;
                cp.ExStyle |= WS_EX_LAYERED;

                return cp;
            }
        }
        private void Shutdown()
        {
            ManagementClass mcOS = new ManagementClass("Win32_OperatingSystem");
            mcOS.Get();
            mcOS.Scope.Options.EnablePrivileges = true;

            // https://learn.microsoft.com/en-us/windows/win32/cimwin32prov/win32shutdown-method-in-class-win32-operatingsystem
            ManagementBaseObject mboShutdownParams = mcOS.GetMethodParameters("Win32Shutdown");
            mboShutdownParams["Reserved"] = "0";
            mboShutdownParams["Flags"] = "5";

            foreach (ManagementObject mo in mcOS.GetInstances())
            {
                mo.InvokeMethod("Win32Shutdown", mboShutdownParams, null);
            }
        }

        public Form1()
        {
            InitializeComponent();
        }

        private DateTime startAt;
        private DateTime endAt;

        private void BeginCountdown(double delay)
        {
            if (startAt.Year != 1)
            {
                Util.MsgBox("There is already a countdown!!");
                return;
            }

            startAt = DateTime.Now;
            endAt = DateTime.Now.AddSeconds(delay);

            // show progress window
            Show();
            WindowState = FormWindowState.Normal;

            // start checking
            timer.Start();
        }

        private void EndCountdown()
        {
            timer.Stop();
            Hide();
            WindowState = FormWindowState.Minimized;
            Shutdown();
        }

        private void startTimer5m_Click(object sender, EventArgs e)
        {
            BeginCountdown(60 * 5);
        }

        private void startTimer10m_Click(object sender, EventArgs e)
        {
            BeginCountdown(60 * 10);
        }

        private void startTimer25m_Click(object sender, EventArgs e)
        {
            BeginCountdown(60 * 25);
        }
        private void startTimer45m_Click(object sender, EventArgs e)
        {
            BeginCountdown(60 * 45);
        }

        private void timer_Tick(object sender, EventArgs e)
        {
            var totalSec = (endAt - startAt).TotalSeconds;
            var remain = (endAt - DateTime.Now);
            var remainSec = remain.TotalSeconds;

            if (remainSec <= 0)
            {
                EndCountdown();
            }

            angle = (float)(remainSec / totalSec * 360);
            timeCircle.Refresh();
        }

        private float angle = 0;

        private void timeCircle_Paint(object sender, PaintEventArgs e)
        {
            var width = 8;

            using var pen = new Pen(Color.ForestGreen, width);

            var size = new Size(timeCircle.Size.Width - width * 2 - 1, timeCircle.Size.Height - width * 2 - 1);
            var point = new Point(width, width);

            var area = new Rectangle(point, size);

            e.Graphics.SmoothingMode = SmoothingMode.HighQuality;
            e.Graphics.DrawArc(pen, area, 270, angle);
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            var bounds = Screen.FromControl(this).Bounds;

            var x = bounds.Width - (int)(bounds.Width * .04) - timeCircle.Width;
            var y = bounds.Height - (int)(bounds.Height * .14) - timeCircle.Height;

            Location = new Point(x, y);
        }

        private void notifyIcon_MouseClick(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                MethodInfo? mi = typeof(NotifyIcon).GetMethod("ShowContextMenu", BindingFlags.Instance | BindingFlags.NonPublic);
                if (mi == null) return;
                mi.Invoke(notifyIcon, null);
            }
        }
    }
}
